package org.wildfly.swarm.config.messaging.activemq.server;

import org.wildfly.swarm.config.runtime.AttributeDocumentation;
import org.wildfly.swarm.config.runtime.ResourceDocumentation;
import org.wildfly.swarm.config.runtime.SingletonResource;
import org.wildfly.swarm.config.runtime.Address;
import org.wildfly.swarm.config.runtime.ResourceType;
import java.beans.PropertyChangeSupport;
import java.beans.PropertyChangeListener;
import org.wildfly.swarm.config.runtime.ModelNodeBinding;
import java.util.Arrays;

/**
 * An address setting defines some attributes that are defined against an
 * address wildcard rather than a specific queue.
 */
@Address("/subsystem=messaging-activemq/server=*/address-setting=*")
@ResourceType("address-setting")
public class AddressSetting<T extends AddressSetting<T>>
		implements
			org.wildfly.swarm.config.runtime.Keyed {

	private String key;
	private PropertyChangeSupport pcs;
	@AttributeDocumentation("Determines what happens when an address where max-size-bytes is specified becomes full. (PAGE, DROP or BLOCK)")
	private AddressFullPolicy addressFullPolicy;
	@AttributeDocumentation("Determines whether ActiveMQ should automatically create an addresses corresponding to the address-settings match when a producer or a consumer is tries to use such a queue.")
	private Boolean autoCreateAddresses;
	@AttributeDocumentation("Determines whether ActiveMQ should automatically create a JMS queue corresponding to the address-settings match when a JMS producer or a consumer is tries to use such a queue.")
	private Boolean autoCreateJmsQueues;
	@AttributeDocumentation("Determines whether ActiveMQ should automatically create a queue corresponding to the address-settings match when a producer or a consumer is tries to use such a queue.")
	private Boolean autoCreateQueues;
	@AttributeDocumentation("Determines Whether ActiveMQ should automatically delete auto-created addresses when they have no consumers and no messages.")
	private Boolean autoDeleteAddresses;
	@AttributeDocumentation("Determines Whether ActiveMQ should automatically delete auto-created JMS queues when they have no consumers and no messages.")
	private Boolean autoDeleteJmsQueues;
	@AttributeDocumentation("Determines Whether ActiveMQ should automatically delete auto-created queues when they have no consumers and no messages.")
	private Boolean autoDeleteQueues;
	@AttributeDocumentation("The dead letter address")
	private String deadLetterAddress;
	@AttributeDocumentation("Defines where to send a message that has expired.")
	private String expiryAddress;
	@AttributeDocumentation("Defines the expiration time that will be used for messages using the default expiration time")
	private Long expiryDelay;
	@AttributeDocumentation("Defines whether a queue only uses last values or not")
	private Boolean lastValueQueue;
	@AttributeDocumentation("Defines how many time a cancelled message can be redelivered before sending to the dead-letter-address")
	private Integer maxDeliveryAttempts;
	@AttributeDocumentation("Maximum value for the redelivery-delay (in ms).")
	private Long maxRedeliveryDelay;
	@AttributeDocumentation("The max bytes size.")
	private Long maxSizeBytes;
	@AttributeDocumentation("Day limit for the message counter history.")
	private Integer messageCounterHistoryDayLimit;
	@AttributeDocumentation("The number of page files to keep in memory to optimize IO during paging navigation.")
	private Integer pageMaxCacheSize;
	@AttributeDocumentation("The paging size.")
	private Integer pageSizeBytes;
	@AttributeDocumentation("Defines how long to wait before attempting redelivery of a cancelled message")
	private Long redeliveryDelay;
	@AttributeDocumentation("Multiplier to apply to the redelivery-delay parameter")
	private Double redeliveryMultiplier;
	@AttributeDocumentation("Defines how long to wait when the last consumer is closed on a queue before redistributing any messages")
	private Long redistributionDelay;
	@AttributeDocumentation("If this parameter is set to true for that address, if the message is not routed to any queues it will instead be sent to the dead letter address (DLA) for that address, if it exists.")
	private Boolean sendToDlaOnNoRoute;
	@AttributeDocumentation("How often to check for slow consumers on a particular queue.")
	private Long slowConsumerCheckPeriod;
	@AttributeDocumentation("Determine what happens when a slow consumer is identified.")
	private SlowConsumerPolicy slowConsumerPolicy;
	@AttributeDocumentation("The minimum rate of message consumption allowed before a consumer is considered slow.")
	private Long slowConsumerThreshold;

	public AddressSetting(java.lang.String key) {
		super();
		this.key = key;
	}

	public String getKey() {
		return this.key;
	}

	/**
	 * Adds a property change listener
	 */
	public void addPropertyChangeListener(PropertyChangeListener listener) {
		if (null == this.pcs)
			this.pcs = new PropertyChangeSupport(this);
		this.pcs.addPropertyChangeListener(listener);
	}

	/**
	 * Removes a property change listener
	 */
	public void removePropertyChangeListener(
			java.beans.PropertyChangeListener listener) {
		if (this.pcs != null)
			this.pcs.removePropertyChangeListener(listener);
	}

	public static enum AddressFullPolicy {
		DROP("DROP"), PAGE("PAGE"), BLOCK("BLOCK"), FAIL("FAIL");
		private final String allowedValue;

		/**
		 * Returns the allowed value for the management model.
		 * 
		 * @return the allowed model value
		 */
		public String getAllowedValue() {
			return allowedValue;
		}

		AddressFullPolicy(java.lang.String allowedValue) {
			this.allowedValue = allowedValue;
		}

		@Override
		public String toString() {
			return allowedValue;
		}
	}

	public static enum SlowConsumerPolicy {
		KILL("KILL"), NOTIFY("NOTIFY");
		private final String allowedValue;

		/**
		 * Returns the allowed value for the management model.
		 * 
		 * @return the allowed model value
		 */
		public String getAllowedValue() {
			return allowedValue;
		}

		SlowConsumerPolicy(java.lang.String allowedValue) {
			this.allowedValue = allowedValue;
		}

		@Override
		public String toString() {
			return allowedValue;
		}
	}

	/**
	 * Determines what happens when an address where max-size-bytes is specified
	 * becomes full. (PAGE, DROP or BLOCK)
	 */
	@ModelNodeBinding(detypedName = "address-full-policy")
	public AddressFullPolicy addressFullPolicy() {
		return this.addressFullPolicy;
	}

	/**
	 * Determines what happens when an address where max-size-bytes is specified
	 * becomes full. (PAGE, DROP or BLOCK)
	 */
	@SuppressWarnings("unchecked")
	public T addressFullPolicy(AddressFullPolicy value) {
		Object oldValue = this.addressFullPolicy;
		this.addressFullPolicy = value;
		if (this.pcs != null)
			this.pcs.firePropertyChange("addressFullPolicy", oldValue, value);
		return (T) this;
	}

	/**
	 * Determines whether ActiveMQ should automatically create an addresses
	 * corresponding to the address-settings match when a producer or a consumer
	 * is tries to use such a queue.
	 */
	@ModelNodeBinding(detypedName = "auto-create-addresses")
	public Boolean autoCreateAddresses() {
		return this.autoCreateAddresses;
	}

	/**
	 * Determines whether ActiveMQ should automatically create an addresses
	 * corresponding to the address-settings match when a producer or a consumer
	 * is tries to use such a queue.
	 */
	@SuppressWarnings("unchecked")
	public T autoCreateAddresses(java.lang.Boolean value) {
		Object oldValue = this.autoCreateAddresses;
		this.autoCreateAddresses = value;
		if (this.pcs != null)
			this.pcs.firePropertyChange("autoCreateAddresses", oldValue, value);
		return (T) this;
	}

	/**
	 * Determines whether ActiveMQ should automatically create a JMS queue
	 * corresponding to the address-settings match when a JMS producer or a
	 * consumer is tries to use such a queue.
	 * 
	 * @deprecated This attribute is no longer supported by ActiveMQ.
	 */
	@Deprecated
	@ModelNodeBinding(detypedName = "auto-create-jms-queues")
	public Boolean autoCreateJmsQueues() {
		return this.autoCreateJmsQueues;
	}

	/**
	 * Determines whether ActiveMQ should automatically create a JMS queue
	 * corresponding to the address-settings match when a JMS producer or a
	 * consumer is tries to use such a queue.
	 * 
	 * @deprecated This attribute is no longer supported by ActiveMQ.
	 */
	@SuppressWarnings("unchecked")
	@Deprecated
	public T autoCreateJmsQueues(java.lang.Boolean value) {
		Object oldValue = this.autoCreateJmsQueues;
		this.autoCreateJmsQueues = value;
		if (this.pcs != null)
			this.pcs.firePropertyChange("autoCreateJmsQueues", oldValue, value);
		return (T) this;
	}

	/**
	 * Determines whether ActiveMQ should automatically create a queue
	 * corresponding to the address-settings match when a producer or a consumer
	 * is tries to use such a queue.
	 */
	@ModelNodeBinding(detypedName = "auto-create-queues")
	public Boolean autoCreateQueues() {
		return this.autoCreateQueues;
	}

	/**
	 * Determines whether ActiveMQ should automatically create a queue
	 * corresponding to the address-settings match when a producer or a consumer
	 * is tries to use such a queue.
	 */
	@SuppressWarnings("unchecked")
	public T autoCreateQueues(java.lang.Boolean value) {
		Object oldValue = this.autoCreateQueues;
		this.autoCreateQueues = value;
		if (this.pcs != null)
			this.pcs.firePropertyChange("autoCreateQueues", oldValue, value);
		return (T) this;
	}

	/**
	 * Determines Whether ActiveMQ should automatically delete auto-created
	 * addresses when they have no consumers and no messages.
	 */
	@ModelNodeBinding(detypedName = "auto-delete-addresses")
	public Boolean autoDeleteAddresses() {
		return this.autoDeleteAddresses;
	}

	/**
	 * Determines Whether ActiveMQ should automatically delete auto-created
	 * addresses when they have no consumers and no messages.
	 */
	@SuppressWarnings("unchecked")
	public T autoDeleteAddresses(java.lang.Boolean value) {
		Object oldValue = this.autoDeleteAddresses;
		this.autoDeleteAddresses = value;
		if (this.pcs != null)
			this.pcs.firePropertyChange("autoDeleteAddresses", oldValue, value);
		return (T) this;
	}

	/**
	 * Determines Whether ActiveMQ should automatically delete auto-created JMS
	 * queues when they have no consumers and no messages.
	 * 
	 * @deprecated This attribute is no longer supported by ActiveMQ.
	 */
	@Deprecated
	@ModelNodeBinding(detypedName = "auto-delete-jms-queues")
	public Boolean autoDeleteJmsQueues() {
		return this.autoDeleteJmsQueues;
	}

	/**
	 * Determines Whether ActiveMQ should automatically delete auto-created JMS
	 * queues when they have no consumers and no messages.
	 * 
	 * @deprecated This attribute is no longer supported by ActiveMQ.
	 */
	@SuppressWarnings("unchecked")
	@Deprecated
	public T autoDeleteJmsQueues(java.lang.Boolean value) {
		Object oldValue = this.autoDeleteJmsQueues;
		this.autoDeleteJmsQueues = value;
		if (this.pcs != null)
			this.pcs.firePropertyChange("autoDeleteJmsQueues", oldValue, value);
		return (T) this;
	}

	/**
	 * Determines Whether ActiveMQ should automatically delete auto-created
	 * queues when they have no consumers and no messages.
	 */
	@ModelNodeBinding(detypedName = "auto-delete-queues")
	public Boolean autoDeleteQueues() {
		return this.autoDeleteQueues;
	}

	/**
	 * Determines Whether ActiveMQ should automatically delete auto-created
	 * queues when they have no consumers and no messages.
	 */
	@SuppressWarnings("unchecked")
	public T autoDeleteQueues(java.lang.Boolean value) {
		Object oldValue = this.autoDeleteQueues;
		this.autoDeleteQueues = value;
		if (this.pcs != null)
			this.pcs.firePropertyChange("autoDeleteQueues", oldValue, value);
		return (T) this;
	}

	/**
	 * The dead letter address
	 */
	@ModelNodeBinding(detypedName = "dead-letter-address")
	public String deadLetterAddress() {
		return this.deadLetterAddress;
	}

	/**
	 * The dead letter address
	 */
	@SuppressWarnings("unchecked")
	public T deadLetterAddress(java.lang.String value) {
		Object oldValue = this.deadLetterAddress;
		this.deadLetterAddress = value;
		if (this.pcs != null)
			this.pcs.firePropertyChange("deadLetterAddress", oldValue, value);
		return (T) this;
	}

	/**
	 * Defines where to send a message that has expired.
	 */
	@ModelNodeBinding(detypedName = "expiry-address")
	public String expiryAddress() {
		return this.expiryAddress;
	}

	/**
	 * Defines where to send a message that has expired.
	 */
	@SuppressWarnings("unchecked")
	public T expiryAddress(java.lang.String value) {
		Object oldValue = this.expiryAddress;
		this.expiryAddress = value;
		if (this.pcs != null)
			this.pcs.firePropertyChange("expiryAddress", oldValue, value);
		return (T) this;
	}

	/**
	 * Defines the expiration time that will be used for messages using the
	 * default expiration time
	 */
	@ModelNodeBinding(detypedName = "expiry-delay")
	public Long expiryDelay() {
		return this.expiryDelay;
	}

	/**
	 * Defines the expiration time that will be used for messages using the
	 * default expiration time
	 */
	@SuppressWarnings("unchecked")
	public T expiryDelay(java.lang.Long value) {
		Object oldValue = this.expiryDelay;
		this.expiryDelay = value;
		if (this.pcs != null)
			this.pcs.firePropertyChange("expiryDelay", oldValue, value);
		return (T) this;
	}

	/**
	 * Defines whether a queue only uses last values or not
	 */
	@ModelNodeBinding(detypedName = "last-value-queue")
	public Boolean lastValueQueue() {
		return this.lastValueQueue;
	}

	/**
	 * Defines whether a queue only uses last values or not
	 */
	@SuppressWarnings("unchecked")
	public T lastValueQueue(java.lang.Boolean value) {
		Object oldValue = this.lastValueQueue;
		this.lastValueQueue = value;
		if (this.pcs != null)
			this.pcs.firePropertyChange("lastValueQueue", oldValue, value);
		return (T) this;
	}

	/**
	 * Defines how many time a cancelled message can be redelivered before
	 * sending to the dead-letter-address
	 */
	@ModelNodeBinding(detypedName = "max-delivery-attempts")
	public Integer maxDeliveryAttempts() {
		return this.maxDeliveryAttempts;
	}

	/**
	 * Defines how many time a cancelled message can be redelivered before
	 * sending to the dead-letter-address
	 */
	@SuppressWarnings("unchecked")
	public T maxDeliveryAttempts(java.lang.Integer value) {
		Object oldValue = this.maxDeliveryAttempts;
		this.maxDeliveryAttempts = value;
		if (this.pcs != null)
			this.pcs.firePropertyChange("maxDeliveryAttempts", oldValue, value);
		return (T) this;
	}

	/**
	 * Maximum value for the redelivery-delay (in ms).
	 */
	@ModelNodeBinding(detypedName = "max-redelivery-delay")
	public Long maxRedeliveryDelay() {
		return this.maxRedeliveryDelay;
	}

	/**
	 * Maximum value for the redelivery-delay (in ms).
	 */
	@SuppressWarnings("unchecked")
	public T maxRedeliveryDelay(java.lang.Long value) {
		Object oldValue = this.maxRedeliveryDelay;
		this.maxRedeliveryDelay = value;
		if (this.pcs != null)
			this.pcs.firePropertyChange("maxRedeliveryDelay", oldValue, value);
		return (T) this;
	}

	/**
	 * The max bytes size.
	 */
	@ModelNodeBinding(detypedName = "max-size-bytes")
	public Long maxSizeBytes() {
		return this.maxSizeBytes;
	}

	/**
	 * The max bytes size.
	 */
	@SuppressWarnings("unchecked")
	public T maxSizeBytes(java.lang.Long value) {
		Object oldValue = this.maxSizeBytes;
		this.maxSizeBytes = value;
		if (this.pcs != null)
			this.pcs.firePropertyChange("maxSizeBytes", oldValue, value);
		return (T) this;
	}

	/**
	 * Day limit for the message counter history.
	 */
	@ModelNodeBinding(detypedName = "message-counter-history-day-limit")
	public Integer messageCounterHistoryDayLimit() {
		return this.messageCounterHistoryDayLimit;
	}

	/**
	 * Day limit for the message counter history.
	 */
	@SuppressWarnings("unchecked")
	public T messageCounterHistoryDayLimit(java.lang.Integer value) {
		Object oldValue = this.messageCounterHistoryDayLimit;
		this.messageCounterHistoryDayLimit = value;
		if (this.pcs != null)
			this.pcs.firePropertyChange("messageCounterHistoryDayLimit",
					oldValue, value);
		return (T) this;
	}

	/**
	 * The number of page files to keep in memory to optimize IO during paging
	 * navigation.
	 */
	@ModelNodeBinding(detypedName = "page-max-cache-size")
	public Integer pageMaxCacheSize() {
		return this.pageMaxCacheSize;
	}

	/**
	 * The number of page files to keep in memory to optimize IO during paging
	 * navigation.
	 */
	@SuppressWarnings("unchecked")
	public T pageMaxCacheSize(java.lang.Integer value) {
		Object oldValue = this.pageMaxCacheSize;
		this.pageMaxCacheSize = value;
		if (this.pcs != null)
			this.pcs.firePropertyChange("pageMaxCacheSize", oldValue, value);
		return (T) this;
	}

	/**
	 * The paging size.
	 */
	@ModelNodeBinding(detypedName = "page-size-bytes")
	public Integer pageSizeBytes() {
		return this.pageSizeBytes;
	}

	/**
	 * The paging size.
	 */
	@SuppressWarnings("unchecked")
	public T pageSizeBytes(java.lang.Integer value) {
		Object oldValue = this.pageSizeBytes;
		this.pageSizeBytes = value;
		if (this.pcs != null)
			this.pcs.firePropertyChange("pageSizeBytes", oldValue, value);
		return (T) this;
	}

	/**
	 * Defines how long to wait before attempting redelivery of a cancelled
	 * message
	 */
	@ModelNodeBinding(detypedName = "redelivery-delay")
	public Long redeliveryDelay() {
		return this.redeliveryDelay;
	}

	/**
	 * Defines how long to wait before attempting redelivery of a cancelled
	 * message
	 */
	@SuppressWarnings("unchecked")
	public T redeliveryDelay(java.lang.Long value) {
		Object oldValue = this.redeliveryDelay;
		this.redeliveryDelay = value;
		if (this.pcs != null)
			this.pcs.firePropertyChange("redeliveryDelay", oldValue, value);
		return (T) this;
	}

	/**
	 * Multiplier to apply to the redelivery-delay parameter
	 */
	@ModelNodeBinding(detypedName = "redelivery-multiplier")
	public Double redeliveryMultiplier() {
		return this.redeliveryMultiplier;
	}

	/**
	 * Multiplier to apply to the redelivery-delay parameter
	 */
	@SuppressWarnings("unchecked")
	public T redeliveryMultiplier(java.lang.Double value) {
		Object oldValue = this.redeliveryMultiplier;
		this.redeliveryMultiplier = value;
		if (this.pcs != null)
			this.pcs.firePropertyChange("redeliveryMultiplier", oldValue, value);
		return (T) this;
	}

	/**
	 * Defines how long to wait when the last consumer is closed on a queue
	 * before redistributing any messages
	 */
	@ModelNodeBinding(detypedName = "redistribution-delay")
	public Long redistributionDelay() {
		return this.redistributionDelay;
	}

	/**
	 * Defines how long to wait when the last consumer is closed on a queue
	 * before redistributing any messages
	 */
	@SuppressWarnings("unchecked")
	public T redistributionDelay(java.lang.Long value) {
		Object oldValue = this.redistributionDelay;
		this.redistributionDelay = value;
		if (this.pcs != null)
			this.pcs.firePropertyChange("redistributionDelay", oldValue, value);
		return (T) this;
	}

	/**
	 * If this parameter is set to true for that address, if the message is not
	 * routed to any queues it will instead be sent to the dead letter address
	 * (DLA) for that address, if it exists.
	 */
	@ModelNodeBinding(detypedName = "send-to-dla-on-no-route")
	public Boolean sendToDlaOnNoRoute() {
		return this.sendToDlaOnNoRoute;
	}

	/**
	 * If this parameter is set to true for that address, if the message is not
	 * routed to any queues it will instead be sent to the dead letter address
	 * (DLA) for that address, if it exists.
	 */
	@SuppressWarnings("unchecked")
	public T sendToDlaOnNoRoute(java.lang.Boolean value) {
		Object oldValue = this.sendToDlaOnNoRoute;
		this.sendToDlaOnNoRoute = value;
		if (this.pcs != null)
			this.pcs.firePropertyChange("sendToDlaOnNoRoute", oldValue, value);
		return (T) this;
	}

	/**
	 * How often to check for slow consumers on a particular queue.
	 */
	@ModelNodeBinding(detypedName = "slow-consumer-check-period")
	public Long slowConsumerCheckPeriod() {
		return this.slowConsumerCheckPeriod;
	}

	/**
	 * How often to check for slow consumers on a particular queue.
	 */
	@SuppressWarnings("unchecked")
	public T slowConsumerCheckPeriod(java.lang.Long value) {
		Object oldValue = this.slowConsumerCheckPeriod;
		this.slowConsumerCheckPeriod = value;
		if (this.pcs != null)
			this.pcs.firePropertyChange("slowConsumerCheckPeriod", oldValue,
					value);
		return (T) this;
	}

	/**
	 * Determine what happens when a slow consumer is identified.
	 */
	@ModelNodeBinding(detypedName = "slow-consumer-policy")
	public SlowConsumerPolicy slowConsumerPolicy() {
		return this.slowConsumerPolicy;
	}

	/**
	 * Determine what happens when a slow consumer is identified.
	 */
	@SuppressWarnings("unchecked")
	public T slowConsumerPolicy(SlowConsumerPolicy value) {
		Object oldValue = this.slowConsumerPolicy;
		this.slowConsumerPolicy = value;
		if (this.pcs != null)
			this.pcs.firePropertyChange("slowConsumerPolicy", oldValue, value);
		return (T) this;
	}

	/**
	 * The minimum rate of message consumption allowed before a consumer is
	 * considered slow.
	 */
	@ModelNodeBinding(detypedName = "slow-consumer-threshold")
	public Long slowConsumerThreshold() {
		return this.slowConsumerThreshold;
	}

	/**
	 * The minimum rate of message consumption allowed before a consumer is
	 * considered slow.
	 */
	@SuppressWarnings("unchecked")
	public T slowConsumerThreshold(java.lang.Long value) {
		Object oldValue = this.slowConsumerThreshold;
		this.slowConsumerThreshold = value;
		if (this.pcs != null)
			this.pcs.firePropertyChange("slowConsumerThreshold", oldValue,
					value);
		return (T) this;
	}
}